from django.db import models

from django.utils.translation import gettext_lazy as g
from django.db.models import SmallIntegerField
from pytz import timezone


class ShiftSlot(models.Model):
    tzinfo = timezone('Europe/Copenhagen')

    start = models.TimeField('start of shift')
    end = models.TimeField('end of shift')
    description = models.CharField(max_length=128)

    def __str__(self):
        return "{0} shift {1}-{2}".format(self.description,
                self.start.isoformat(timespec='minutes'),
                self.end.isoformat(timespec='minutes'))

    class Meta:
        ordering = ['start', 'end']

class WeeklyShift(models.Model):
    DAY_OF_THE_WEEK = (
        (0, g('Monday')),
        (1, g('Tuesday')),
        (2, g('Wednesday')),
        (3, g('Thursday')),
        (4, g('Friday')),
        (5, g('Saturday')),
        (6, g('Sunday')),
    )
    day_of_the_week = models.PositiveSmallIntegerField(choices=DAY_OF_THE_WEEK)
    shift_slot = models.ForeignKey(ShiftSlot, on_delete=models.CASCADE)

    def __str__(self):
        return "{0} {1}".format(self.get_day_of_the_week_display(),
                self.shift_slot)

    class Meta:
        ordering = ['day_of_the_week', 'shift_slot']


class CleaningTask(models.Model):
    shift = models.ForeignKey(WeeklyShift, on_delete=models.CASCADE)
    task_description = models.TextField('cleaning task description')
    active = models.BooleanField(
            verbose_name='active task',
            default=True)

    def __str__(self):
        return "{state}{shift}\n{description}".format(
                state='<INACTIVE>' if not self.active else '',
                shift=self.shift,
                description=self.task_description)

    class Meta:
        ordering = ['-active', 'shift']

class CleaningTaskSignoff(models.Model):
    task = models.ForeignKey(CleaningTask, on_delete=models.PROTECT)
    date = models.DateTimeField(
            verbose_name = 'sign-off date',
            auto_now_add = True,
            blank = True,
    )
    employee = models.CharField(max_length=255, verbose_name='employee')

    def __str__(self):
        return "{0} at {1} for {2}".format(
                self.employee,
                self.date.isoformat(timespec='minutes'),
                self.task)

class LogEntry(models.Model):
    author = models.CharField(
            blank=True,
            max_length=255,
            verbose_name='author',
    )
    log_text = models.TextField(max_length=8192)
    pub_date = models.DateTimeField('date published')

    def __str__(self):
        return self.log_text

    class Meta:
        verbose_name_plural = 'log entries'
